"""Client for the model registry."""

from __future__ import annotations

from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from dataclasses import dataclass
from typing import TypeVar, cast

from mr_openapi import (
    ApiClient,
    Configuration,
    ModelRegistryServiceApi,
)
from mr_openapi import (
    exceptions as mr_exceptions,
)
from typing_extensions import overload

from ._utils import required_args
from .types import (
    Artifact,
    ListOptions,
    ModelArtifact,
    ModelVersion,
    RegisteredModel,
)

ArtifactT = TypeVar("ArtifactT", bound=Artifact)


@dataclass
class ModelRegistryAPIClient:
    """Model registry API."""

    config: Configuration

    @classmethod
    def secure_connection(
        cls,
        server_address: str,
        port: int = 443,
        *,
        user_token: bytes,
        custom_ca: str | None = None,
    ) -> ModelRegistryAPIClient:
        """Constructor.

        Args:
            server_address: Server address.
            port: Server port. Defaults to 443.

        Keyword Args:
            user_token: The PEM-encoded user token as a byte string.
            custom_ca: The path to a PEM-
        """
        return cls(
            Configuration(
                f"{server_address}:{port}",
                access_token=user_token,
                ssl_ca_cert=custom_ca,
            )
        )

    @classmethod
    def insecure_connection(
        cls,
        server_address: str,
        port: int,
        user_token: bytes | None = None,
    ) -> ModelRegistryAPIClient:
        """Constructor.

        Args:
            server_address: Server address.
            port: Server port.
            user_token: The PEM-encoded user token as a byte string.
        """
        return cls(
            Configuration(host=f"{server_address}:{port}", access_token=user_token)
        )

    @asynccontextmanager
    async def get_client(self) -> AsyncIterator[ModelRegistryServiceApi]:
        """Get a client for the model registry."""
        api_client = ApiClient(self.config)
        client = ModelRegistryServiceApi(api_client)

        try:
            yield client
        finally:
            await api_client.close()

    async def upsert_registered_model(
        self, registered_model: RegisteredModel
    ) -> RegisteredModel:
        """Upsert a registered model.

        Updates or creates a registered model on the server.

        Args:
            registered_model: Registered model.

        Returns:
            New registered model.
        """
        async with self.get_client() as client:
            if registered_model.id:
                rm = await client.update_registered_model(
                    registered_model.id, registered_model.update()
                )
            else:
                rm = await client.create_registered_model(registered_model.create())

        return RegisteredModel.from_basemodel(rm)

    async def get_registered_model_by_id(self, id: str) -> RegisteredModel | None:
        """Fetch a registered model by its ID.

        Args:
            id: Registered model ID.

        Returns:
            Registered model.
        """
        async with self.get_client() as client:
            try:
                rm = await client.get_registered_model(id)
            except mr_exceptions.NotFoundException:
                return None

        return RegisteredModel.from_basemodel(rm)

    @overload
    async def get_registered_model_by_params(self, name: str): ...

    @overload
    async def get_registered_model_by_params(self, *, external_id: str): ...

    @required_args(("name",), ("external_id",))
    async def get_registered_model_by_params(
        self, name: str | None = None, external_id: str | None = None
    ) -> RegisteredModel | None:
        """Fetch a registered model by its name or external ID.

        Args:
            name: Registered model name.
            external_id: Registered model external ID.

        Returns:
            Registered model.
        """
        async with self.get_client() as client:
            try:
                rm = await client.find_registered_model(
                    name=name, external_id=external_id
                )
            except mr_exceptions.NotFoundException:
                return None

        return RegisteredModel.from_basemodel(rm)

    async def get_registered_models(
        self, options: ListOptions | None = None
    ) -> list[RegisteredModel]:
        """Fetch registered models.

        Args:
            options: Options for listing registered models.

        Returns:
            Registered models.
        """
        async with self.get_client() as client:
            rm_list = await client.get_registered_models(
                **(options or ListOptions()).as_options()
            )

        if options:
            options.next_page_token = rm_list.next_page_token

        return [RegisteredModel.from_basemodel(rm) for rm in rm_list.items or []]

    async def upsert_model_version(
        self, model_version: ModelVersion, registered_model_id: str
    ) -> ModelVersion:
        """Upsert a model version.

        Updates or creates a model version on the server.

        Args:
            model_version: Model version to upsert.
            registered_model_id: ID of the registered model this version will be associated to.

        Returns:
            New model version.
        """
        async with self.get_client() as client:
            if model_version.id:
                mv = await client.update_model_version(
                    model_version.id, model_version.update()
                )
            else:
                mv = await client.create_model_version(
                    model_version.create(registered_model_id=registered_model_id)
                )

        return ModelVersion.from_basemodel(mv)

    async def get_model_version_by_id(
        self, model_version_id: str
    ) -> ModelVersion | None:
        """Fetch a model version by its ID.

        Args:
            model_version_id: Model version ID.

        Returns:
            Model version.
        """
        async with self.get_client() as client:
            try:
                mv = await client.get_model_version(model_version_id)
            except mr_exceptions.NotFoundException:
                return None

        return ModelVersion.from_basemodel(mv)

    async def get_model_versions(
        self, registered_model_id: str, options: ListOptions | None = None
    ) -> list[ModelVersion]:
        """Fetch model versions by registered model ID.

        Args:
            registered_model_id: Registered model ID.
            options: Options for listing model versions.

        Returns:
            Model versions.
        """
        async with self.get_client() as client:
            mv_list = await client.get_registered_model_versions(
                registered_model_id, **(options or ListOptions()).as_options()
            )

        if options:
            options.next_page_token = mv_list.next_page_token

        return [ModelVersion.from_basemodel(mv) for mv in mv_list.items or []]

    @overload
    async def get_model_version_by_params(
        self, registered_model_id: str, name: str
    ): ...

    @overload
    async def get_model_version_by_params(self, *, external_id: str): ...

    @required_args(
        (
            "registered_model_id",
            "name",
        ),
        ("external_id",),
    )
    async def get_model_version_by_params(
        self,
        registered_model_id: str | None = None,
        name: str | None = None,
        external_id: str | None = None,
    ) -> ModelVersion | None:
        """Fetch a model version by associated parameters.

        Either fetches by using external ID or by using registered model ID and version name.

        Args:
            registered_model_id: Registered model ID.
            name: Model version.
            external_id: Model version external ID.

        Returns:
            Model version.
        """
        async with self.get_client() as client:
            try:
                mv = await client.find_model_version(
                    name=name,
                    external_id=external_id,
                    parent_resource_id=registered_model_id,
                )
            except mr_exceptions.NotFoundException:
                return None

        return ModelVersion.from_basemodel(mv)

    async def upsert_model_artifact(
        self, model_artifact: ModelArtifact, model_version_id: str
    ) -> ModelArtifact:
        """Upsert a model artifact.

        Updates or creates a model artifact on the server.

        Args:
            model_artifact: Model artifact to upsert.
            model_version_id: ID of the model version this artifact will be associated to.

        Returns:
            New model artifact.
        """
        if not model_artifact.id:
            return await self.create_model_version_artifact(
                model_artifact, model_version_id
            )

        async with self.get_client() as client:
            return ModelArtifact.from_basemodel(
                await client.update_model_artifact(
                    model_artifact.id, model_artifact.update()
                )
            )

    async def create_model_version_artifact(
        self, artifact: ArtifactT, model_version_id: str
    ) -> ArtifactT:
        """Creates a model version artifact.

        Creates a model version artifact on the server.

        Args:
            artifact: Model version artifact to upsert.
            model_version_id: ID of the model version this artifact will be associated to.

        Returns:
            New model version artifact.
        """
        async with self.get_client() as client:
            return cast(
                ArtifactT,
                Artifact.validate_artifact(
                    await client.create_model_version_artifact(
                        model_version_id, artifact.wrap()
                    )
                ),
            )

    async def get_model_artifact_by_id(self, id: str) -> ModelArtifact | None:
        """Fetch a model artifact by its ID.

        Args:
            id: Model artifact ID.

        Returns:
            Model artifact.
        """
        async with self.get_client() as client:
            try:
                ma = await client.get_model_artifact(id)
            except mr_exceptions.NotFoundException:
                return None

        return ModelArtifact.from_basemodel(ma)

    @overload
    async def get_model_artifact_by_params(
        self,
        name: str,
        model_version_id: str,
    ): ...

    @overload
    async def get_model_artifact_by_params(self, *, external_id: str): ...

    @required_args(
        (
            "name",
            "model_version_id",
        ),
        ("external_id",),
    )
    async def get_model_artifact_by_params(
        self,
        name: str | None = None,
        model_version_id: str | None = None,
        external_id: str | None = None,
    ) -> ModelArtifact | None:
        """Fetch a model artifact either by external ID or by its name and the ID of its associated model version.

        Args:
            name: Model artifact name.
            model_version_id: ID of the associated model version.
            external_id: Model artifact external ID.

        Returns:
            Model artifact.
        """
        async with self.get_client() as client:
            try:
                ma = await client.find_model_artifact(
                    name=name,
                    parent_resource_id=model_version_id,
                    external_id=external_id,
                )
            except mr_exceptions.NotFoundException:
                return None

        return ModelArtifact.from_basemodel(ma)

    async def get_model_artifacts(
        self,
        model_version_id: str | None = None,
        options: ListOptions | None = None,
    ) -> list[ModelArtifact]:
        """Fetches model artifacts.

        Args:
            model_version_id: ID of the associated model version.
            options: Options for listing model artifacts.

        Returns:
            Model artifacts.
        """
        async with self.get_client() as client:
            if model_version_id:
                art_list = await client.get_model_version_artifacts(
                    model_version_id, **(options or ListOptions()).as_options()
                )
                if options:
                    options.next_page_token = art_list.next_page_token
                models = []
                for art in art_list.items or []:
                    converted = Artifact.validate_artifact(art)
                    if isinstance(converted, ModelArtifact):
                        models.append(converted)
                return models

            ma_list = await client.get_model_artifacts(
                **(options or ListOptions()).as_options()
            )
            if options:
                options.next_page_token = ma_list.next_page_token
            return [ModelArtifact.from_basemodel(ma) for ma in ma_list.items or []]

    async def get_model_version_artifacts(
        self,
        model_version_id: str,
        options: ListOptions | None = None,
    ) -> list[Artifact]:
        """Fetches model artifacts.

        Args:
            model_version_id: ID of the associated model version.
            options: Options for listing model artifacts.

        Returns:
            Model artifacts.
        """
        async with self.get_client() as client:
            art_list = await client.get_model_version_artifacts(
                model_version_id, **(options or ListOptions()).as_options()
            )
        if options:
            options.next_page_token = art_list.next_page_token
        return [Artifact.validate_artifact(art) for art in art_list.items or []]



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from mr_openapi.models.model_version_create import ModelVersionCreate


class TestModelVersionCreate(unittest.TestCase):
    """ModelVersionCreate unit test stubs."""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ModelVersionCreate:
        """Test ModelVersionCreate
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included.
        """
        # uncomment below to create an instance of `ModelVersionCreate`
        """
        model = ModelVersionCreate()
        if include_optional:
            return ModelVersionCreate(
                custom_properties = {
                    'key' : null
                    },
                description = '',
                external_id = '',
                name = '',
                state = 'LIVE',
                author = '',
                registered_model_id = ''
            )
        else:
            return ModelVersionCreate(
                registered_model_id = '',
        )
        """

    def testModelVersionCreate(self):
        """Test ModelVersionCreate."""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()



"""Exceptions for the model registry."""


class StoreError(Exception):
    """Storage related error."""


class MissingMetadata(Exception):
    """Not enough metadata to complete operation."""


class UnsupportedType(StoreError):
    """Raised when an unsupported type is encountered."""


class TypeNotFound(StoreError):
    """Raised when a type cannot be found."""


class ServerError(StoreError):
    """Raised when the server returns a bad response."""


class DuplicateError(StoreError):
    """Raised when the user tries to put an object with a conflicting property."""



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from mr_openapi.api.model_registry_service_api import ModelRegistryServiceApi


class TestModelRegistryServiceApi(unittest.TestCase):
    """ModelRegistryServiceApi unit test stubs."""

    def setUp(self) -> None:
        self.api = ModelRegistryServiceApi()

    def tearDown(self) -> None:
        pass

    def test_create_environment_inference_service(self) -> None:
        """Test case for create_environment_inference_service.

        Create a InferenceService in ServingEnvironment
        """
        pass

    def test_create_inference_service(self) -> None:
        """Test case for create_inference_service.

        Create a InferenceService
        """
        pass

    def test_create_inference_service_serve(self) -> None:
        """Test case for create_inference_service_serve.

        Create a ServeModel action in a InferenceService
        """
        pass

    def test_create_model_artifact(self) -> None:
        """Test case for create_model_artifact.

        Create a ModelArtifact
        """
        pass

    def test_create_model_version(self) -> None:
        """Test case for create_model_version.

        Create a ModelVersion
        """
        pass

    def test_create_model_version_artifact(self) -> None:
        """Test case for create_model_version_artifact.

        Create an Artifact in a ModelVersion
        """
        pass

    def test_create_registered_model(self) -> None:
        """Test case for create_registered_model.

        Create a RegisteredModel
        """
        pass

    def test_create_registered_model_version(self) -> None:
        """Test case for create_registered_model_version.

        Create a ModelVersion in RegisteredModel
        """
        pass

    def test_create_serving_environment(self) -> None:
        """Test case for create_serving_environment.

        Create a ServingEnvironment
        """
        pass

    def test_find_inference_service(self) -> None:
        """Test case for find_inference_service.

        Get an InferenceServices that matches search parameters.
        """
        pass

    def test_find_model_artifact(self) -> None:
        """Test case for find_model_artifact.

        Get a ModelArtifact that matches search parameters.
        """
        pass

    def test_find_model_version(self) -> None:
        """Test case for find_model_version.

        Get a ModelVersion that matches search parameters.
        """
        pass

    def test_find_registered_model(self) -> None:
        """Test case for find_registered_model.

        Get a RegisteredModel that matches search parameters.
        """
        pass

    def test_find_serving_environment(self) -> None:
        """Test case for find_serving_environment.

        Find ServingEnvironment
        """
        pass

    def test_get_environment_inference_services(self) -> None:
        """Test case for get_environment_inference_services.

        List All ServingEnvironment's InferenceServices
        """
        pass

    def test_get_inference_service(self) -> None:
        """Test case for get_inference_service.

        Get a InferenceService
        """
        pass

    def test_get_inference_service_model(self) -> None:
        """Test case for get_inference_service_model.

        Get InferenceService's RegisteredModel
        """
        pass

    def test_get_inference_service_serves(self) -> None:
        """Test case for get_inference_service_serves.

        List All InferenceService's ServeModel actions
        """
        pass

    def test_get_inference_service_version(self) -> None:
        """Test case for get_inference_service_version.

        Get InferenceService's ModelVersion
        """
        pass

    def test_get_inference_services(self) -> None:
        """Test case for get_inference_services.

        List All InferenceServices
        """
        pass

    def test_get_model_artifact(self) -> None:
        """Test case for get_model_artifact.

        Get a ModelArtifact
        """
        pass

    def test_get_model_artifacts(self) -> None:
        """Test case for get_model_artifacts.

        List All ModelArtifacts
        """
        pass

    def test_get_model_version(self) -> None:
        """Test case for get_model_version.

        Get a ModelVersion
        """
        pass

    def test_get_model_version_artifacts(self) -> None:
        """Test case for get_model_version_artifacts.

        List all artifacts associated with the `ModelVersion`
        """
        pass

    def test_get_model_versions(self) -> None:
        """Test case for get_model_versions.

        List All ModelVersions
        """
        pass

    def test_get_registered_model(self) -> None:
        """Test case for get_registered_model.

        Get a RegisteredModel
        """
        pass

    def test_get_registered_model_versions(self) -> None:
        """Test case for get_registered_model_versions.

        List All RegisteredModel's ModelVersions
        """
        pass

    def test_get_registered_models(self) -> None:
        """Test case for get_registered_models.

        List All RegisteredModels
        """
        pass

    def test_get_serving_environment(self) -> None:
        """Test case for get_serving_environment.

        Get a ServingEnvironment
        """
        pass

    def test_get_serving_environments(self) -> None:
        """Test case for get_serving_environments.

        List All ServingEnvironments
        """
        pass

    def test_update_inference_service(self) -> None:
        """Test case for update_inference_service.

        Update a InferenceService
        """
        pass

    def test_update_model_artifact(self) -> None:
        """Test case for update_model_artifact.

        Update a ModelArtifact
        """
        pass

    def test_update_model_version(self) -> None:
        """Test case for update_model_version.

        Update a ModelVersion
        """
        pass

    def test_update_registered_model(self) -> None:
        """Test case for update_registered_model.

        Update a RegisteredModel
        """
        pass

    def test_update_serving_environment(self) -> None:
        """Test case for update_serving_environment.

        Update a ServingEnvironment
        """
        pass


if __name__ == "__main__":
    unittest.main()



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class OrderByField(str, Enum):
    """Supported fields for ordering result entities."""

    """
    allowed enum values
    """
    CREATE_TIME = "CREATE_TIME"
    LAST_UPDATE_TIME = "LAST_UPDATE_TIME"
    ID = "Id"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of OrderByField from a JSON string."""
        return cls(json.loads(json_str))



# coding: utf-8

# flake8: noqa

"""
    Model Registry REST API

    REST API for Model Registry to create and manage ML model metadata

    The version of the OpenAPI document: v1alpha3
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from mr_openapi.api.model_registry_service_api import ModelRegistryServiceApi

# import ApiClient
from mr_openapi.api_response import ApiResponse
from mr_openapi.api_client import ApiClient
from mr_openapi.configuration import Configuration
from mr_openapi.exceptions import OpenApiException
from mr_openapi.exceptions import ApiTypeError
from mr_openapi.exceptions import ApiValueError
from mr_openapi.exceptions import ApiKeyError
from mr_openapi.exceptions import ApiAttributeError
from mr_openapi.exceptions import ApiException

# import models into sdk package
from mr_openapi.models.artifact import Artifact
from mr_openapi.models.artifact_list import ArtifactList
from mr_openapi.models.artifact_state import ArtifactState
from mr_openapi.models.base_artifact import BaseArtifact
from mr_openapi.models.base_artifact_create import BaseArtifactCreate
from mr_openapi.models.base_artifact_update import BaseArtifactUpdate
from mr_openapi.models.base_execution import BaseExecution
from mr_openapi.models.base_execution_create import BaseExecutionCreate
from mr_openapi.models.base_execution_update import BaseExecutionUpdate
from mr_openapi.models.base_resource import BaseResource
from mr_openapi.models.base_resource_create import BaseResourceCreate
from mr_openapi.models.base_resource_list import BaseResourceList
from mr_openapi.models.base_resource_update import BaseResourceUpdate
from mr_openapi.models.doc_artifact import DocArtifact
from mr_openapi.models.error import Error
from mr_openapi.models.execution_state import ExecutionState
from mr_openapi.models.inference_service import InferenceService
from mr_openapi.models.inference_service_create import InferenceServiceCreate
from mr_openapi.models.inference_service_list import InferenceServiceList
from mr_openapi.models.inference_service_state import InferenceServiceState
from mr_openapi.models.inference_service_update import InferenceServiceUpdate
from mr_openapi.models.metadata_bool_value import MetadataBoolValue
from mr_openapi.models.metadata_double_value import MetadataDoubleValue
from mr_openapi.models.metadata_int_value import MetadataIntValue
from mr_openapi.models.metadata_proto_value import MetadataProtoValue
from mr_openapi.models.metadata_string_value import MetadataStringValue
from mr_openapi.models.metadata_struct_value import MetadataStructValue
from mr_openapi.models.metadata_value import MetadataValue
from mr_openapi.models.model_artifact import ModelArtifact
from mr_openapi.models.model_artifact_create import ModelArtifactCreate
from mr_openapi.models.model_artifact_list import ModelArtifactList
from mr_openapi.models.model_artifact_update import ModelArtifactUpdate
from mr_openapi.models.model_version import ModelVersion
from mr_openapi.models.model_version_create import ModelVersionCreate
from mr_openapi.models.model_version_list import ModelVersionList
from mr_openapi.models.model_version_state import ModelVersionState
from mr_openapi.models.model_version_update import ModelVersionUpdate
from mr_openapi.models.order_by_field import OrderByField
from mr_openapi.models.registered_model import RegisteredModel
from mr_openapi.models.registered_model_create import RegisteredModelCreate
from mr_openapi.models.registered_model_list import RegisteredModelList
from mr_openapi.models.registered_model_state import RegisteredModelState
from mr_openapi.models.registered_model_update import RegisteredModelUpdate
from mr_openapi.models.serve_model import ServeModel
from mr_openapi.models.serve_model_create import ServeModelCreate
from mr_openapi.models.serve_model_list import ServeModelList
from mr_openapi.models.serve_model_update import ServeModelUpdate
from mr_openapi.models.serving_environment import ServingEnvironment
from mr_openapi.models.serving_environment_create import ServingEnvironmentCreate
from mr_openapi.models.serving_environment_list import ServingEnvironmentList
from mr_openapi.models.serving_environment_update import ServingEnvironmentUpdate
from mr_openapi.models.sort_order import SortOrder



"""Artifact types for model registry.

Artifacts represent pieces of data.
This could be datasets, models, metrics, or any other piece of data produced or consumed by an
execution, such as an experiment run.

Those types are used to map between proto types based on artifacts and Python objects.

Todo:
    * Move part of the description to API Reference docs (#120).
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any, TypeVar

from mr_openapi import (
    Artifact as ArtifactBaseModel,
)
from mr_openapi import (
    ArtifactState,
    ModelArtifactCreate,
    ModelArtifactUpdate,
)
from mr_openapi import (
    DocArtifact as DocArtifactBaseModel,
)
from mr_openapi import (
    ModelArtifact as ModelArtifactBaseModel,
)
from typing_extensions import override

from .base import BaseResourceModel

A = TypeVar("A", bound="Artifact")


class Artifact(BaseResourceModel, ABC):
    """Base class for all artifacts.

    Attributes:
        name: Name of the artifact.
        uri: URI of the artifact.
        state: State of the artifact.
    """

    name: str | None = None
    uri: str
    state: ArtifactState = ArtifactState.UNKNOWN

    @classmethod
    def from_artifact(cls: type[A], source: ArtifactBaseModel) -> A:
        """Convert a base artifact."""
        model = source.actual_instance
        assert model
        return cls.from_basemodel(model)

    @staticmethod
    def validate_artifact(source: ArtifactBaseModel) -> DocArtifact | ModelArtifact:
        """Validate an artifact."""
        model = source.actual_instance
        assert model
        if isinstance(model, DocArtifactBaseModel):
            return DocArtifact.from_basemodel(model)
        return ModelArtifact.from_basemodel(model)

    @abstractmethod
    def as_basemodel(self) -> Any:
        """Wrap the object in a BaseModel object."""

    def wrap(self) -> ArtifactBaseModel:
        """Wrap the object in a ArtifactBaseModel object."""
        return ArtifactBaseModel(self.as_basemodel())


class DocArtifact(Artifact):
    """Represents a Document Artifact.

    Attributes:
        name: Name of the document.
        uri: URI of the document.
        description: Description of the object.
        external_id: Customizable ID. Has to be unique among instances of the same type.
    """

    @override
    def create(self, **kwargs) -> Any:
        raise NotImplementedError

    @override
    def update(self, **kwargs) -> Any:
        raise NotImplementedError

    @override
    def as_basemodel(self) -> DocArtifactBaseModel:
        return DocArtifactBaseModel(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("custom_properties")),
            artifactType="doc-artifact",
        )

    @classmethod
    @override
    def from_basemodel(cls, source: DocArtifactBaseModel) -> DocArtifact:
        assert source.name
        assert source.uri
        assert source.state
        return cls(
            id=source.id,
            name=source.name,
            description=source.description,
            external_id=source.external_id,
            create_time_since_epoch=source.create_time_since_epoch,
            last_update_time_since_epoch=source.last_update_time_since_epoch,
            uri=source.uri,
            state=source.state,
            custom_properties=cls._unmap_custom_properties(source.custom_properties)
            if source.custom_properties
            else None,
        )


class ModelArtifact(Artifact):
    """Represents a Model.

    Attributes:
        name: Name of the model.
        uri: URI of the model.
        description: Description of the object.
        external_id: Customizable ID. Has to be unique among instances of the same type.
        model_format_name: Name of the model format.
        model_format_version: Version of the model format.
        storage_key: Storage secret name.
        storage_path: Storage path of the model.
        service_account_name: Name of the service account with storage secret.
    """

    # TODO: this could be an enum of valid formats
    model_format_name: str | None = None
    model_format_version: str | None = None
    storage_key: str | None = None
    storage_path: str | None = None
    service_account_name: str | None = None

    _model_version_id: str | None = None

    @override
    def create(self, **kwargs) -> ModelArtifactCreate:
        """Create a new ModelArtifactCreate object."""
        return ModelArtifactCreate(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("id", "custom_properties")),
            **kwargs,
        )

    @override
    def update(self, **kwargs) -> ModelArtifactUpdate:
        """Create a new ModelArtifactUpdate object."""
        return ModelArtifactUpdate(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("id", "name", "custom_properties")),
            **kwargs,
        )

    @override
    def as_basemodel(self) -> ModelArtifactBaseModel:
        return ModelArtifactBaseModel(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("custom_properties")),
            artifactType="model-artifact",
        )

    @classmethod
    @override
    def from_basemodel(cls, source: ModelArtifactBaseModel) -> ModelArtifact:
        """Create a new ModelArtifact object from a BaseModel object."""
        assert source.name
        assert source.uri
        assert source.state
        return cls(
            id=source.id,
            name=source.name,
            description=source.description,
            external_id=source.external_id,
            create_time_since_epoch=source.create_time_since_epoch,
            last_update_time_since_epoch=source.last_update_time_since_epoch,
            uri=source.uri,
            model_format_name=source.model_format_name,
            model_format_version=source.model_format_version,
            storage_key=source.storage_key,
            storage_path=source.storage_path,
            service_account_name=source.service_account_name,
            state=source.state,
            custom_properties=cls._unmap_custom_properties(source.custom_properties)
            if source.custom_properties
            else None,
        )



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest


class TestRegisteredModelState(unittest.TestCase):
    """RegisteredModelState unit test stubs."""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testRegisteredModelState(self):
        """Test RegisteredModelState."""
        # inst = RegisteredModelState()


if __name__ == "__main__":
    unittest.main()



"""Context types for model registry.

Contexts group related Artifacts together.
They provide a way to organize and categorize components in a workflow.

Those types are used to map between proto types based on contexts and Python objects.

Todo:
    * Move part of the description to API Reference docs (#120).
"""

from __future__ import annotations

from mr_openapi import (
    ModelVersion as ModelVersionBaseModel,
)
from mr_openapi import (
    ModelVersionCreate,
    ModelVersionState,
    ModelVersionUpdate,
    RegisteredModelCreate,
    RegisteredModelState,
    RegisteredModelUpdate,
)
from mr_openapi import (
    RegisteredModel as RegisteredModelBaseModel,
)
from typing_extensions import override

from .base import BaseResourceModel


class ModelVersion(BaseResourceModel):
    """Represents a model version.

    Attributes:
        name: Name of this version.
        author: Author of the model version.
        description: Description of the object.
        external_id: Customizable ID. Has to be unique among instances of the same type.
        artifacts: Artifacts associated with this version.
    """

    name: str
    author: str | None = None
    state: ModelVersionState = ModelVersionState.LIVE

    @override
    def create(self, *, registered_model_id: str, **kwargs) -> ModelVersionCreate:  # type: ignore[override]
        return ModelVersionCreate(
            registeredModelId=registered_model_id,
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("id", "custom_properties")),
            **kwargs,
        )

    @override
    def update(self, **kwargs) -> ModelVersionUpdate:
        return ModelVersionUpdate(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("id", "name", "custom_properties")),
            **kwargs,
        )

    @classmethod
    @override
    def from_basemodel(cls, source: ModelVersionBaseModel) -> ModelVersion:
        assert source.name
        assert source.state
        return cls(
            id=source.id,
            name=source.name,
            state=source.state,
            author=source.author,
            description=source.description,
            external_id=source.external_id,
            create_time_since_epoch=source.create_time_since_epoch,
            last_update_time_since_epoch=source.last_update_time_since_epoch,
            custom_properties=cls._unmap_custom_properties(source.custom_properties)
            if source.custom_properties
            else None,
        )


class RegisteredModel(BaseResourceModel):
    """Represents a registered model.

    Attributes:
        name: Registered model name.
        owner: Owner of this Registered Model.
        description: Description of the object.
        external_id: Customizable ID. Has to be unique among instances of the same type.
    """

    name: str
    owner: str | None = None
    state: RegisteredModelState = RegisteredModelState.LIVE

    @override
    def create(self, **kwargs) -> RegisteredModelCreate:
        return RegisteredModelCreate(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("id", "custom_properties")),
            **kwargs,
        )

    @override
    def update(self, **kwargs) -> RegisteredModelUpdate:
        return RegisteredModelUpdate(
            customProperties=self._map_custom_properties(),
            **self._props_as_dict(exclude=("id", "name", "custom_properties")),
            **kwargs,
        )

    @classmethod
    @override
    def from_basemodel(cls, source: RegisteredModelBaseModel) -> RegisteredModel:
        assert source.name
        assert source.state
        return cls(
            id=source.id,
            name=source.name,
            owner=source.owner,
            state=source.state,
            description=source.description,
            external_id=source.external_id,
            create_time_since_epoch=source.create_time_since_epoch,
            last_update_time_since_epoch=source.last_update_time_since_epoch,
            custom_properties=cls._unmap_custom_properties(source.custom_properties)
            if source.custom_properties
            else None,
        )



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class InferenceServiceState(str, Enum):
    """- DEPLOYED: A state indicating that the `InferenceService` should be deployed. - UNDEPLOYED: A state indicating that the `InferenceService` should be un-deployed. The state indicates the desired state of inference service. See the associated `ServeModel` for the actual status of service deployment action."""

    """
    allowed enum values
    """
    DEPLOYED = "DEPLOYED"
    UNDEPLOYED = "UNDEPLOYED"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of InferenceServiceState from a JSON string."""
        return cls(json.loads(json_str))



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from mr_openapi.models.serving_environment_list import ServingEnvironmentList


class TestServingEnvironmentList(unittest.TestCase):
    """ServingEnvironmentList unit test stubs."""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ServingEnvironmentList:
        """Test ServingEnvironmentList
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included.
        """
        # uncomment below to create an instance of `ServingEnvironmentList`
        """
        model = ServingEnvironmentList()
        if include_optional:
            return ServingEnvironmentList(
                next_page_token = '',
                page_size = 56,
                size = 56,
                items = [
                    null
                    ]
            )
        else:
            return ServingEnvironmentList(
                next_page_token = '',
                page_size = 56,
                size = 56,
        )
        """

    def testServingEnvironmentList(self):
        """Test ServingEnvironmentList."""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class ExecutionState(str, Enum):
    """The state of the Execution. The state transitions are   NEW -> RUNNING -> COMPLETE | CACHED | FAILED | CANCELED CACHED means the execution is skipped due to cached results. CANCELED means the execution is skipped due to precondition not met. It is different from CACHED in that a CANCELED execution will not have any event associated with it. It is different from FAILED in that there is no unexpected error happened and it is regarded as a normal state.  See also: ml-metadata Execution.State."""

    """
    allowed enum values
    """
    UNKNOWN = "UNKNOWN"
    NEW = "NEW"
    RUNNING = "RUNNING"
    COMPLETE = "COMPLETE"
    FAILED = "FAILED"
    CACHED = "CACHED"
    CANCELED = "CANCELED"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ExecutionState from a JSON string."""
        return cls(json.loads(json_str))



"""Base types for model registry."""

from __future__ import annotations

from abc import ABC, abstractmethod
from collections.abc import Sequence
from typing import Any, Union, get_args

from mr_openapi.models.metadata_value import MetadataValue
from pydantic import BaseModel, ConfigDict

SupportedTypes = Union[bool, int, float, str]


class BaseResourceModel(BaseModel, ABC):
    """Abstract base type for protos.

    This is a type defining common functionality for all types representing Model Registry resources,
    such as Artifacts, Contexts, and Executions.

    Attributes:
        id: Object ID. Auto-assigned when put on the server.
        name: Name of the object.
        description: Description of the object.
        external_id: Customizable ID. Has to be unique among instances of the same type.
        create_time_since_epoch: Seconds elapsed since object creation time, measured against epoch.
        last_update_time_since_epoch: Seconds elapsed since object last update time, measured against epoch.
    """

    model_config = ConfigDict(protected_namespaces=())

    id: str | None = None
    description: str | None = None
    external_id: str | None = None
    create_time_since_epoch: str | None = None
    last_update_time_since_epoch: str | None = None
    custom_properties: dict[str, SupportedTypes] | None = None

    @abstractmethod
    def create(self, **kwargs) -> Any:
        """Convert the object to a create request."""

    @abstractmethod
    def update(self, **kwargs) -> Any:
        """Convert the object to an update request."""

    @classmethod
    @abstractmethod
    def from_basemodel(cls, source: Any) -> Any:
        """Create a new object from a BaseModel object."""

    def _map_custom_properties(
        self,
    ) -> dict[str, MetadataValue] | None:
        """Map properties from Python to proto.

        Args:
            py_props: Python properties.
            mlmd_props: Proto properties, will be modified in place.
        """
        if not self.custom_properties:
            return None

        def get_meta_type(v: SupportedTypes) -> str:
            if isinstance(v, float):
                return "double"
            if isinstance(v, str):
                return "string"
            return type(v).__name__.lower()

        def get_meta_value(v: SupportedTypes) -> MetadataValue:
            type = get_meta_type(v)
            v = str(v) if isinstance(v, int) and not isinstance(v, bool) else v
            return MetadataValue.from_dict(
                {
                    f"{type}_value": v,
                    "metadataType": f"Metadata{type.capitalize()}Value",
                }
            )

        dest = {}
        for key, value in self.custom_properties.items():
            if value is None:
                continue
            dest[key] = get_meta_value(value)
        return dest

    @classmethod
    def _unmap_custom_properties(
        cls, custom_properties: dict[str, MetadataValue]
    ) -> dict[str, SupportedTypes]:
        def get_meta_value(meta: Any) -> SupportedTypes:
            type_name = meta.metadata_type[8:-5].lower()
            # Metadata type names are in the format Metadata<Type>Value
            v = getattr(meta, f"{type_name}_value")
            if type_name == "int":
                return int(v)
            return v

        return {
            name: value
            for name, meta_value in custom_properties.items()
            if isinstance(
                value := get_meta_value(meta_value.actual_instance),
                get_args(SupportedTypes),
            )
        }

    def _props_as_dict(
        self, exclude: Sequence[str] | None = None, alias: bool = False
    ) -> dict[str, Any]:
        exclude = exclude or []
        return {
            k: getattr(self, k)
            for k in self.model_json_schema(alias).get("properties", {})
            if k not in exclude
        }



"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class ArtifactState(str, Enum):
    """- PENDING: A state indicating that the artifact may exist.  - LIVE: A state indicating that the artifact should exist, unless something external to the system deletes it.  - MARKED_FOR_DELETION: A state indicating that the artifact should be deleted.  - DELETED: A state indicating that the artifact has been deleted.  - ABANDONED: A state indicating that the artifact has been abandoned, which may be due to a failed or cancelled execution.  - REFERENCE: A state indicating that the artifact is a reference artifact. At execution start time, the orchestrator produces an output artifact for each output key with state PENDING. However, for an intermediate artifact, this first artifact's state will be REFERENCE. Intermediate artifacts emitted during a component's execution will copy the REFERENCE artifact's attributes. At the end of an execution, the artifact state should remain REFERENCE instead of being changed to LIVE.  See also: ml-metadata Artifact.State."""

    """
    allowed enum values
    """
    UNKNOWN = "UNKNOWN"
    PENDING = "PENDING"
    LIVE = "LIVE"
    MARKED_FOR_DELETION = "MARKED_FOR_DELETION"
    DELETED = "DELETED"
    ABANDONED = "ABANDONED"
    REFERENCE = "REFERENCE"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ArtifactState from a JSON string."""
        return cls(json.loads(json_str))
